#include "mainwindow.h"

#include <QCameraInfo>
#include <QDebug>

#include <QCameraViewfinderSettings>
#include <QMediaMetaData>

MainWindow::MainWindow(QWidget *parent) : QMainWindow(parent), camera(NULL), cameraInfo(NULL), viewfinder(NULL), captureImage(NULL), captureVideo(NULL)
{
    viewfinder = new QCameraViewfinder(this);

    if (QCameraInfo::availableCameras().count() > 0)
    {
        qDebug() << "Caméra(s) disponible(s)" << QCameraInfo::availableCameras().count();
        QList<QCameraInfo> cameras = QCameraInfo::availableCameras();
        foreach (const QCameraInfo &_cameraInfo, cameras)
        {
            qDebug() << "Device" << _cameraInfo.deviceName();
            qDebug() << "Description" << _cameraInfo.description();
            if (_cameraInfo.position() == QCamera::FrontFace)
                qDebug() << "Position : FrontFace";
            else if (_cameraInfo.position() == QCamera::BackFace)
                qDebug() << "Position : BackFace";
            qDebug() << "Orientation" << _cameraInfo.orientation() << "degrés";
        }
        //camera = new QCamera;
        camera = new QCamera("/dev/video0", this);
        cameraInfo = new QCameraInfo(*camera);
        this->statusBar()->showMessage(cameraInfo->description() + " [" + cameraInfo->deviceName() + "]");
        connect(camera, SIGNAL(statusChanged(QCamera::Status)), this, SLOT(statusCamera(QCamera::Status)));
        connect(camera, SIGNAL(stateChanged(QCamera::State)), this, SLOT(etatCamera(QCamera::State)));
        camera->setViewfinder(viewfinder);

        //QCameraViewfinderSettings viewfinderSettings;
        //viewfinderSettings.setResolution(2592, 1944);
        //viewfinderSettings.setResolution(640, 480);
        //viewfinderSettings.setMinimumFrameRate(15.0);
        //viewfinderSettings.setMaximumFrameRate(30.0);
        //viewfinderSettings.setResolution(1920, 1080);
        //viewfinderSettings.setResolution(1024, 768);
        //viewfinderSettings.setMaximumFrameRate(5.0);
        //viewfinderSettings.setMaximumFrameRate(10.0);
        //viewfinderSettings.setPixelFormat(QVideoFrame::Format_YUYV);
        //camera->setViewfinderSettings(viewfinderSettings);

        /*viewfinderSettings = camera->viewfinderSettings();
        qDebug() << "Resolution" << viewfinderSettings.resolution();
        qDebug() << "FrameRate min" << viewfinderSettings.minimumFrameRate();
        qDebug() << "FrameRate max" << viewfinderSettings.maximumFrameRate();
        qDebug() << "Format" << viewfinderSettings.pixelFormat();
        camera->load();
        qDebug() << "Resolutions" << camera->supportedViewfinderResolutions();*/
    }
    else
    {
        qDebug() << "Aucune caméra disponible !";
    }

    captureImage = new QCameraImageCapture(camera, this);
    qDebug() << "Capture Image available" << captureImage->isAvailable() << "CaptureToBuffer" << captureImage->isCaptureDestinationSupported(QCameraImageCapture::CaptureToBuffer);
    qDebug() << "Capture Image codecs" << captureImage->supportedImageCodecs();
    qDebug() << "Capture Image resolution(s)" << captureImage->supportedResolutions();
    connect(captureImage, SIGNAL(readyForCaptureChanged(bool)), this, SLOT(estPretCaptureImage(bool)));
    connect(captureImage, SIGNAL(imageCaptured(int,QImage)), this, SLOT(imageCaptured(int,QImage)));
    connect(captureImage, SIGNAL(imageSaved(int,QString)), this, SLOT(imageSauvegardee(int,QString)));

    captureVideo = new QMediaRecorder(camera, this);
    qDebug() << "Capture Video available" << captureVideo->isAvailable();
    qDebug() << "Capture Video codecs" << captureVideo->supportedVideoCodecs();
    qDebug() << "Capture Video resolution(s)" << captureVideo->supportedResolutions();
    qDebug() << "Capture Video frame rate(s)" << captureVideo->supportedFrameRates();
    qDebug() << "Capture Video containers" << captureVideo->supportedContainers();

    captureVideo->setMetaData(QMediaMetaData::Title, QVariant(QLatin1String("Test Camera USB")));
    captureVideo->setOutputLocation(qApp->applicationDirPath());
    connect(captureVideo, SIGNAL(stateChanged(QMediaRecorder::State)), this, SLOT(etatCaptureVideo(QMediaRecorder::State)));
    connect(captureVideo, SIGNAL(durationChanged(qint64)), this, SLOT(afficherTempsEnregistrement(qint64)));
    //connect(captureVideo, SIGNAL(), this, SLOT());
    enregistre = false;

    camera->setCaptureMode(QCamera::CaptureStillImage);
    qDebug() << "Capture Mode " << camera->captureMode() << "CaptureStillImage";

    // GUI
    QWidget *widgetPrincipal = new QWidget;
    QVBoxLayout *layoutPrincipal = new QVBoxLayout;
    QHBoxLayout *layoutFlux = new QHBoxLayout;
    viewfinder->setFixedSize(640, 480);
    layoutFlux->addWidget(viewfinder);
    QHBoxLayout *layoutBoutons = new QHBoxLayout;
    QPushButton *boutonImage = new QPushButton(QString::fromUtf8("Image"), this);
    connect(boutonImage, SIGNAL(clicked(bool)), this, SLOT(enregistrerImage()));
    QPushButton *boutonVideo = new QPushButton(QString::fromUtf8("Vidéo"), this);
    connect(boutonVideo, SIGNAL(clicked(bool)), this, SLOT(enregistrerVideo()));
    QComboBox *listeTypeCapture = new QComboBox(this);
    connect(listeTypeCapture, SIGNAL(currentIndexChanged(int)), this, SLOT(selectionnerCapture(int)));
    listeTypeCapture->addItem(QString::fromUtf8("Image"));
    listeTypeCapture->addItem(QString::fromUtf8("Vidéo"));
    layoutBoutons->addStretch();
    layoutBoutons->addWidget(listeTypeCapture);
    layoutBoutons->addWidget(boutonImage);
    layoutBoutons->addWidget(boutonVideo);
    layoutPrincipal->addLayout(layoutFlux);
    layoutPrincipal->addLayout(layoutBoutons);
    widgetPrincipal->setLayout(layoutPrincipal);

    setCentralWidget(widgetPrincipal);
    //setCentralWidget(viewfinder);
    setWindowTitle(QString::fromUtf8("Test Caméra USB"));

    if(camera != NULL)
        camera->start();
}

MainWindow::~MainWindow()
{
    if(camera != NULL)
        camera->stop();
}

void MainWindow::etatCamera(QCamera::State etat)
{
    qDebug() << Q_FUNC_INFO << etat;
}

void MainWindow::statusCamera(QCamera::Status etat)
{
    qDebug() << Q_FUNC_INFO << etat;
}

void MainWindow::etatCaptureVideo(QMediaRecorder::State etat)
{
    qDebug() << Q_FUNC_INFO << etat;
}

void MainWindow::enregistrerImage()
{
    if(camera == NULL)
        return;
    if(camera->captureMode() == QCamera::CaptureStillImage)
    {
        qDebug() << Q_FUNC_INFO;
        camera->searchAndLock();
        captureImage->capture(qApp->applicationDirPath());
        camera->unlock();
    }
}

void MainWindow::enregistrerVideo()
{
    if(camera == NULL)
        return;
    if(camera->captureMode() == QCamera::CaptureVideo)
    {
        qDebug() << Q_FUNC_INFO << enregistre;
        if(!enregistre)
        {
            qDebug() << Q_FUNC_INFO << "record()";
            this->statusBar()->showMessage(QString::fromUtf8("Enregistrement vidéo dans ") + captureVideo->outputLocation().toString());
            captureVideo->record();
            enregistre = true;
        }
        else if (captureVideo->state() == QMediaRecorder::RecordingState)
        {
            qDebug() << Q_FUNC_INFO << "stop()";
            captureVideo->stop();
            enregistre = false;
            this->statusBar()->showMessage(QString::fromUtf8("Arrêt enregistrement vidéo [") + captureVideo->outputLocation().toString() + "]", 2000);
        }
    }
}

void MainWindow::afficherTempsEnregistrement(qint64 temps)
{
    QString str = QString::fromUtf8("Enregistrement vidéo %1 s").arg(captureVideo->duration()/1000);
    qDebug() << Q_FUNC_INFO << temps << "ms" << str;
    this->statusBar()->showMessage(str);
}

void MainWindow::estPretCaptureImage(bool pret)
{
    qDebug() << Q_FUNC_INFO << pret;
}

void MainWindow::imageCaptured(int id, QImage image)
{
    Q_UNUSED(image)
    qDebug() << Q_FUNC_INFO << id;
}

void MainWindow::imageSauvegardee(int id, QString nomFichier)
{
    qDebug() << Q_FUNC_INFO << id << QDir::toNativeSeparators(nomFichier);
    this->statusBar()->showMessage("Image " + nomFichier + QString::fromUtf8(" sauvegardée"), 2000);
}

void MainWindow::selectionnerCapture(int index)
{
    switch (index)
    {
    case 0:
        camera->setCaptureMode(QCamera::CaptureStillImage);
        break;
    case 1:
        camera->setCaptureMode(QCamera::CaptureVideo);
        break;
    default:
        break;
    }

    if(camera->captureMode() == QCamera::CaptureStillImage)
        qDebug() << "Capture Mode " << camera->captureMode() << "CaptureStillImage";
    else if(camera->captureMode() == QCamera::CaptureVideo)
        qDebug() << "Capture Mode " << camera->captureMode() << "CaptureVideo";
    else
        qDebug() << "Capture Mode " << camera->captureMode();

    if(cameraInfo != NULL)
        this->statusBar()->showMessage(cameraInfo->description() + " [" + cameraInfo->deviceName() + "]");
}
